##############################################################################
## FUNCTION NAME: your.graph.new
##
## DESCRIPTION: This function generates summary graphs for a given gender and
## cause of disease.  It takes in as input either a file or an object
## generated by your.cast(). Alternatively, any data that is formatted in the
## manner described below can be used as the input. It then uses function
## plot.cast.new() to generate upto 2 graphs for every country.
##
##           graph 1: Time series of the data and the forecasts, one for each
##                    age group.
##
##           graph 2: Age profiles, both observed and predicted, one for each
##                    year.
##
## IMPORTED FUNCTIONS: 
##                     long.disease
##                     plot.cast
## INPUT:
##      input.data: Usually the output from your.cast(), which could either be
##                  an output file (e.g. output_allc_g2_modelOLS.dat) or an
##                  object (from a command such as yourcast.object <- your.cast())
##
##        yourcast: Logical variable telling the function whether or not your.cast()
##                  was used to generate the data.  (default: F)
##    dv.transform: a transformation to be performed on the data. (default: function(x){x})
##
## The next four parameters tells the program specifically which data to plot.
## The corresponding data is listed below:
##    obs.insample: observed data for the in-sample period (default:T)
##   pred.insample: predicted data for the in-sample period (default:T)
##   obs.outsample: observed data for the out-sample period (default:T)
##  pred.outsample: observed data for the out-sample period (default:T)
##
##The remaining are all parameters relating to the layout of the graphs.
##
##plot.time.series: Logical variable telling the function whether or
##                  not to plot the time series graph. (default: T)
##
##plot.age.profiles: Logical variable telling the function whether or
##                   not to plot the age profiles graph. (default: T)
##
##            wide: width of the graphics region in the postscript file. 
##                  (default: 8)
##
##            high: height of the graphics region in the postscript file. 
##                  (default is 8)
##
## page.horizontal: logical input specifying orientation of the image in the postscript file.
##                  (default:F)
##
##         row.num: number of rows into which graphs should be organized. (default: 1)
##
##         col.num: number of columns into which graphs should be organized. (default: 2)
##
##  background.col: background color for the postscript file. (default: "tan1")
##
##      print2file: logical input specifying whether output should be printed to
##                  an .eps file or to the screen. (default:.eps file)
##
##
##  NOTE: the user can provide her own mortality data for a given cause and
##  gender in the format of a file or a list.  The following elements should be
##  present in the file or list.
##
##  plotting .data: a list where item is a M x N matrix of log-mortality
##                  rates for a given country, where M and N correspond
##                  with years and age groups respectively.  The name of
##                  each item should be the country name as it should appear
##                  in the labels of the graphs.
##
##        depvar : a character string specifying the cause of death
##
##       strata   : "m" for male and "f" for female
##     cntry.lst : a vector that stores the country names
##
## WRITTEN BY:  Federico Girosi and Nirmala Ravishankar
##              CBRSS, Harvard University
##
## last revised : Thursday, September 22nd, 2004
##
##############################################################################
yourgraph.new <- function(input.data, yourcast = F, 
                          obs.insample= T, obs.outsample = T,
                          pred.insample = T, pred.outsample = T,
                          wide = 8, high = 8, plot.time.series = T,
                          plot.age.profiles = T, page.horizontal = FALSE,
                          row.num = 1, col.num = 2, background.col = "tan1",
                          print2file = T, dv.transform = function(x){x}){
  
  ##Creating directory path name
  
  dirpath <- paste(getwd(), "/OUTPUT/", sep = "")
 
  ## Next we load the input file, which is usually the output from your.cast().
  ## It can either be a data file or an object. If it is not generated
  ## from your.cast(), then it should be configured as per the specifications
  ## of the yourcast object.
  data.vec <-  c("cntry.lst", "depvar", "strata", "insampy", "outsampy",
                 "yhatout", "yhatin", "model")
  
  if (is.character(input.data)){
    print("Reading in data file.")
    file <- paste(dirpath, input.data, sep="")
    load(file)
   
  } else{
    print("Obtaining data object")
    if (any(is.element(data.vec, names(input.data))==F)){
      stop("Not enough data. Provide cntry.lst, depvar, strata,
             insampy, outsampy, yhatout, yhatin, model")}

    cntry.lst <- input.data$cntry.lst
    depvar <- input.data$depvar
    strata <- input.data$strata
    insampy <- input.data$insampy
    outsampy <- input.data$outsampy
    yhatout <- input.data$yhatout
    yhatin <- input.data$yhatin
    model <- input.data$model
  }
    data.sym <- sapply(data.vec,as.symbol)
    ev <- environment()
    check <- sapply(data.sym, eval, env=ev)
  
  insampy <- lapply(insampy,FUN = dv.transform) 
  outsampy <- lapply(outsampy,FUN = dv.transform)
  yhatin <- lapply(yhatin,FUN = dv.transform)
  yhatout <- lapply(lapply(yhatout, na.omit), FUN = dv.transform)
 
  
 ## Creating disease, gender and country name strings for labels
  lname <- long.causes(depvar)
  disease.name <- lname$name
  countries <- as.numeric(names(cntry.lst))
  c.name <- unlist(cntry.lst)
  
  if (is.numeric(strata)){
    if (strata==2) {
      gender.str <- "m"}
    else {
      gender.str <- "f"}
  } else gender.str <- strata


  ##The following does the actual plotting using plot.cast.new()
  if (sum(pred.insample == T, pred.outsample == T, obs.insample == T, obs.outsample ==T) == 0){
    ## The above checks if there is any data to plot.
    print("No data found for plotting.")}
  else {
    if (is.logical(print2file)){
      if(print2file == T){
        ## Creating the output file
        print("Creating output file")
        outfilename <- paste("trends_", depvar, "_", gender.str,
                             "_model", model, ".eps", sep = "")
        outfile <- paste(dirpath, outfilename, sep = "")
        
        postscript(outfile,paper="letter",pointsize = 14,width=wide,
                   height=high,horizontal= page.horizontal)
      }
    } else {
      if (!is.character(print2file)) stop("print2file must be either logical or character")
      print("Creating output file with customized name")      
      outfilename <- print2file
      outfile <- paste(dirpath, outfilename,".eps", sep = "")
        postscript(outfile,paper="letter",pointsize = 14,width=wide,
                   height=high,horizontal= page.horizontal)      
    }
    
    par(mfrow = c(row.num,col.num))
    par(cex.main = 1.4, cex.lab = 1.4, cex.axis = 1.4,font=2,font.axis=2,font.lab=2)
    par(mar = c(4, 5, 2, 1))
    
    ## The following code extracts the data for each country, formats it and then
    ## uses plot.new.graph() to plot the selected graphs.
    
    for (i in 1:length(countries)){
      
      pred.in <- yhatin[[i]]
      pred.out <- yhatout[[i]]
      obs.in <- insampy[[i]]
      obs.out <- outsampy[[i]]
      
      ##
      cntry.str <- c.name[i]
      ## Setting graphing parameters like background color, layout of the page 
     
      par(bg= background.col);
      plotcast.new(pred.in, pred.out, obs.in, obs.out, disease.name, gender.str,
                cntry.str, print2file,plot.time.series,pred.outsample,
                   pred.insample,obs.outsample,obs.insample, plot.age.profiles)
      
    }
    dev.off()
  } 
   
}



  
    
